@@ -1,10 +1,10 @@
+import datetime
 import json
 
 from psycopg2.extras import DateRange, DateTimeTZRange, NumericRange, Range
 
 from django.contrib.postgres import forms, lookups
 from django.db import models
-from django.utils import six
 
 from .utils import AttributeSetter
 
@@ -17,6 +17,24 @@ __all__ = [
 class RangeField(models.Field):
     empty_strings_allowed = False
 
+    def __init__(self, *args, **kwargs):
+        # Initializing base_field here ensures that its model matches the model for self.
+        if hasattr(self, 'base_field'):
+            self.base_field = self.base_field()
+        super().__init__(*args, **kwargs)
+
+    @property
+    def model(self):
+        try:
+            return self.__dict__['model']
+        except KeyError:
+            raise AttributeError("'%s' object has no attribute 'model'" % self.__class__.__name__)
+
+    @model.setter
+    def model(self, model):
+        self.__dict__['model'] = model
+        self.base_field.model = model
+
     def get_prep_value(self, value):
         if value is None:
             return None
@@ -27,7 +45,7 @@ class RangeField(models.Field):
         return value
 
     def to_python(self, value):
-        if isinstance(value, six.string_types):
+        if isinstance(value, str):
             # Assume we're deserializing
             vals = json.loads(value)
             for end in ('lower', 'upper'):
@@ -39,7 +57,7 @@ class RangeField(models.Field):
         return value
 
     def set_attributes_from_name(self, name):
-        super(RangeField, self).set_attributes_from_name(name)
+        super().set_attributes_from_name(name)
         self.base_field.set_attributes_from_name(name)
 
     def value_to_string(self, obj):
@@ -61,11 +79,11 @@ class RangeField(models.Field):
 
     def formfield(self, **kwargs):
         kwargs.setdefault('form_class', self.form_field)
-        return super(RangeField, self).formfield(**kwargs)
+        return super().formfield(**kwargs)
 
 
 class IntegerRangeField(RangeField):
-    base_field = models.IntegerField()
+    base_field = models.IntegerField
     range_type = NumericRange
     form_field = forms.IntegerRangeField
 
@@ -74,7 +92,7 @@ class IntegerRangeField(RangeField):
 
 
 class BigIntegerRangeField(RangeField):
-    base_field = models.BigIntegerField()
+    base_field = models.BigIntegerField
     range_type = NumericRange
     form_field = forms.IntegerRangeField
 
@@ -83,7 +101,7 @@ class BigIntegerRangeField(RangeField):
 
 
 class FloatRangeField(RangeField):
-    base_field = models.FloatField()
+    base_field = models.FloatField
     range_type = NumericRange
     form_field = forms.FloatRangeField
 
@@ -92,7 +110,7 @@ class FloatRangeField(RangeField):
 
 
 class DateTimeRangeField(RangeField):
-    base_field = models.DateTimeField()
+    base_field = models.DateTimeField
     range_type = DateTimeTZRange
     form_field = forms.DateTimeRangeField
 
@@ -101,7 +119,7 @@ class DateTimeRangeField(RangeField):
 
 
 class DateRangeField(RangeField):
-    base_field = models.DateField()
+    base_field = models.DateField
     range_type = DateRange
     form_field = forms.DateRangeField
 
@@ -114,6 +132,37 @@ RangeField.register_lookup(lookups.ContainedBy)
 RangeField.register_lookup(lookups.Overlap)
 
 
+class DateTimeRangeContains(models.Lookup):
+    """
+    Lookup for Date/DateTimeRange containment to cast the rhs to the correct
+    type.
+    """
+    lookup_name = 'contains'
+
+    def process_rhs(self, compiler, connection):
+        # Transform rhs value for db lookup.
+        if isinstance(self.rhs, datetime.date):
+            output_field = models.DateTimeField() if isinstance(self.rhs, datetime.datetime) else models.DateField()
+            value = models.Value(self.rhs, output_field=output_field)
+            self.rhs = value.resolve_expression(compiler.query)
+        return super().process_rhs(compiler, connection)
+
+    def as_sql(self, compiler, connection):
+        lhs, lhs_params = self.process_lhs(compiler, connection)
+        rhs, rhs_params = self.process_rhs(compiler, connection)
+        params = lhs_params + rhs_params
+        # Cast the rhs if needed.
+        cast_sql = ''
+        if isinstance(self.rhs, models.Expression) and self.rhs._output_field_or_none:
+            cast_internal_type = self.lhs.output_field.base_field.get_internal_type()
+            cast_sql = '::{}'.format(connection.data_types.get(cast_internal_type))
+        return '%s @> %s%s' % (lhs, rhs, cast_sql), params
+
+
+DateRangeField.register_lookup(DateTimeRangeContains)
+DateTimeRangeField.register_lookup(DateTimeRangeContains)
+
+
 class RangeContainedBy(models.Lookup):
     lookup_name = 'contained_by'
     type_mapping = {
@@ -136,7 +185,7 @@ class RangeContainedBy(models.Lookup):
         return sql % (lhs, rhs), params
 
     def get_prep_lookup(self):
-        return RangeField().get_prep_lookup(self.lookup_name, self.rhs)
+        return RangeField().get_prep_value(self.rhs)
 
 
 models.DateField.register_lookup(RangeContainedBy)
