@@ -1,5 +1,3 @@
-from __future__ import unicode_literals
-
 import base64
 import logging
 import string
@@ -12,7 +10,7 @@ from django.utils import timezone
 from django.utils.crypto import (
     constant_time_compare, get_random_string, salted_hmac,
 )
-from django.utils.encoding import force_bytes, force_text
+from django.utils.encoding import force_bytes
 from django.utils.module_loading import import_string
 
 # session_key should not be case sensitive because some backends can store it
@@ -35,7 +33,7 @@ class UpdateError(Exception):
     pass
 
 
-class SessionBase(object):
+class SessionBase:
     """
     Base class for all Session classes.
     """
@@ -94,7 +92,7 @@ class SessionBase(object):
         return salted_hmac(key_salt, value).hexdigest()
 
     def encode(self, session_dict):
-        "Returns the given session dictionary serialized and encoded as a string."
+        "Return the given session dictionary serialized and encoded as a string."
         serialized = self.serializer().dumps(session_dict)
         hash = self._hash(serialized)
         return base64.b64encode(hash.encode() + b":" + serialized).decode('ascii')
@@ -114,7 +112,7 @@ class SessionBase(object):
             # these happen, just return an empty dictionary (an empty session).
             if isinstance(e, SuspiciousOperation):
                 logger = logging.getLogger('django.security.%s' % e.__class__.__name__)
-                logger.warning(force_text(e))
+                logger.warning(str(e))
             return {}
 
     def update(self, dict_):
@@ -133,15 +131,6 @@ class SessionBase(object):
     def items(self):
         return self._session.items()
 
-    def iterkeys(self):
-        return self._session.iterkeys()
-
-    def itervalues(self):
-        return self._session.itervalues()
-
-    def iteritems(self):
-        return self._session.iteritems()
-
     def clear(self):
         # To avoid unnecessary persistent storage accesses, we set up the
         # internals directly (loading data wastes time, since we are going to
@@ -151,14 +140,14 @@ class SessionBase(object):
         self.modified = True
 
     def is_empty(self):
-        "Returns True when there is no session_key and the session is empty"
+        "Return True when there is no session_key and the session is empty."
         try:
             return not bool(self._session_key) and not self._session_cache
         except AttributeError:
             return True
 
     def _get_new_session_key(self):
-        "Returns session key that isn't being used."
+        "Return session key that isn't being used."
         while True:
             session_key = get_random_string(32, VALID_KEY_CHARS)
             if not self.exists(session_key):
@@ -194,8 +183,8 @@ class SessionBase(object):
 
     def _get_session(self, no_load=False):
         """
-        Lazily loads session from storage (unless "no_load" is True, when only
-        an empty dict is stored) and stores it in the current instance.
+        Lazily load session from storage (unless "no_load" is True, when only
+        an empty dict is stored) and store it in the current instance.
         """
         self.accessed = True
         try:
@@ -258,7 +247,7 @@ class SessionBase(object):
 
     def set_expiry(self, value):
         """
-        Sets a custom expiration for the session. ``value`` can be an integer,
+        Set a custom expiration for the session. ``value`` can be an integer,
         a Python ``datetime`` or ``timedelta`` object or ``None``.
 
         If ``value`` is an integer, the session will expire after that many
@@ -284,7 +273,7 @@ class SessionBase(object):
 
     def get_expire_at_browser_close(self):
         """
-        Returns ``True`` if the session is set to expire when the browser
+        Return ``True`` if the session is set to expire when the browser
         closes, and ``False`` if there's an expiry date. Use
         ``get_expiry_date()`` or ``get_expiry_age()`` to find the actual expiry
         date/age, if there is one.
@@ -295,7 +284,7 @@ class SessionBase(object):
 
     def flush(self):
         """
-        Removes the current session data from the database and regenerates the
+        Remove the current session data from the database and regenerate the
         key.
         """
         self.clear()
@@ -304,7 +293,7 @@ class SessionBase(object):
 
     def cycle_key(self):
         """
-        Creates a new session key, while retaining the current session data.
+        Create a new session key, while retaining the current session data.
         """
         data = self._session
         key = self.session_key
@@ -317,13 +306,13 @@ class SessionBase(object):
 
     def exists(self, session_key):
         """
-        Returns True if the given session_key already exists.
+        Return True if the given session_key already exists.
         """
         raise NotImplementedError('subclasses of SessionBase must provide an exists() method')
 
     def create(self):
         """
-        Creates a new session instance. Guaranteed to create a new object with
+        Create a new session instance. Guaranteed to create a new object with
         a unique key and will have saved the result once (with empty data)
         before the method returns.
         """
@@ -331,23 +320,22 @@ class SessionBase(object):
 
     def save(self, must_create=False):
         """
-        Saves the session data. If 'must_create' is True, a new session object
-        is created (otherwise a CreateError exception is raised). Otherwise,
-        save() only updates an existing object and does not create one
-        (an UpdateError is raised).
+        Save the session data. If 'must_create' is True, create a new session
+        object (or raise CreateError). Otherwise, only update an existing
+        object and don't create one (raise UpdateError if needed).
         """
         raise NotImplementedError('subclasses of SessionBase must provide a save() method')
 
     def delete(self, session_key=None):
         """
-        Deletes the session data under this key. If the key is None, the
-        current session key value is used.
+        Delete the session data under this key. If the key is None, use the
+        current session key value.
         """
         raise NotImplementedError('subclasses of SessionBase must provide a delete() method')
 
     def load(self):
         """
-        Loads the session data and returns a dictionary.
+        Load the session data and return a dictionary.
         """
         raise NotImplementedError('subclasses of SessionBase must provide a load() method')
 
