@@ -1,7 +1,4 @@
-from __future__ import unicode_literals
-
 import copy
-import warnings
 from collections import OrderedDict
 from contextlib import contextmanager
 
@@ -13,9 +10,6 @@ from django.db.models.fields.proxy import OrderWrt
 from django.db.models.fields.related import RECURSIVE_RELATIONSHIP_CONSTANT
 from django.db.models.options import DEFAULT_NAMES, normalize_together
 from django.db.models.utils import make_model_tuple
-from django.utils import six
-from django.utils.deprecation import RemovedInDjango20Warning
-from django.utils.encoding import force_text
 from django.utils.functional import cached_property
 from django.utils.module_loading import import_string
 from django.utils.version import get_docs_version
@@ -24,24 +18,22 @@ from .exceptions import InvalidBasesError
 
 
 def _get_app_label_and_model_name(model, app_label=''):
-    if isinstance(model, six.string_types):
+    if isinstance(model, str):
         split = model.split('.', 1)
-        return (tuple(split) if len(split) == 2 else (app_label, split[0]))
+        return tuple(split) if len(split) == 2 else (app_label, split[0])
     else:
         return model._meta.app_label, model._meta.model_name
 
 
 def _get_related_models(m):
-    """
-    Return all models that have a direct relationship to the given model.
-    """
+    """Return all models that have a direct relationship to the given model."""
     related_models = [
         subclass for subclass in m.__subclasses__()
         if issubclass(subclass, models.Model)
     ]
     related_fields_models = set()
     for f in m._meta.get_fields(include_parents=True, include_hidden=True):
-        if f.is_relation and f.related_model is not None and not isinstance(f.related_model, six.string_types):
+        if f.is_relation and f.related_model is not None and not isinstance(f.related_model, str):
             related_fields_models.add(f.model)
             related_models.append(f.related_model)
     # Reverse accessors of foreign keys to proxy models are attached to their
@@ -85,11 +77,11 @@ def get_related_models_recursive(model):
     return seen - {(model._meta.app_label, model._meta.model_name)}
 
 
-class ProjectState(object):
+class ProjectState:
     """
-    Represents the entire project's overall state.
-    This is the item that is passed around - we do it here rather than at the
-    app level so that cross-app FKs/etc. resolve properly.
+    Represent the entire project's overall state. This is the item that is
+    passed around - do it here rather than at the app level so that cross-app
+    FKs/etc. resolve properly.
     """
 
     def __init__(self, models=None, real_apps=None):
@@ -199,7 +191,7 @@ class ProjectState(object):
         self.apps.render_multiple(states_to_be_rendered)
 
     def clone(self):
-        "Returns an exact copy of this ProjectState"
+        """Return an exact copy of this ProjectState."""
         new_state = ProjectState(
             models={k: v.clone() for k, v in self.models.items()},
             real_apps=self.real_apps,
@@ -224,7 +216,7 @@ class ProjectState(object):
 
     @classmethod
     def from_apps(cls, apps):
-        "Takes in an Apps and returns a ProjectState matching it"
+        """Take an Apps and return a ProjectState matching it."""
         app_models = {}
         for model in apps.get_models(include_swapped=True):
             model_state = ModelState.from_model(model)
@@ -232,20 +224,11 @@ class ProjectState(object):
         return cls(app_models)
 
     def __eq__(self, other):
-        if set(self.models.keys()) != set(other.models.keys()):
-            return False
-        if set(self.real_apps) != set(other.real_apps):
-            return False
-        return all(model == other.models[key] for key, model in self.models.items())
-
-    def __ne__(self, other):
-        return not (self == other)
+        return self.models == other.models and set(self.real_apps) == set(other.real_apps)
 
 
 class AppConfigStub(AppConfig):
-    """
-    Stubs a Django AppConfig. Only provides a label, and a dict of models.
-    """
+    """Stub of an AppConfig. Only provides a label and a dict of models."""
     # Not used, but required by AppConfig.__init__
     path = ''
 
@@ -254,7 +237,7 @@ class AppConfigStub(AppConfig):
         # App-label and app-name are not the same thing, so technically passing
         # in the label here is wrong. In practice, migrations don't care about
         # the app name, but we need something unique, and the label works fine.
-        super(AppConfigStub, self).__init__(label, None)
+        super().__init__(label, None)
 
     def import_models(self):
         self.models = self.apps.all_models[self.label]
@@ -279,7 +262,7 @@ class StateApps(Apps):
         # Populate the app registry with a stub for each application.
         app_labels = {model_state.app_label for model_state in models.values()}
         app_configs = [AppConfigStub(label) for label in sorted(real_apps + list(app_labels))]
-        super(StateApps, self).__init__(app_configs)
+        super().__init__(app_configs)
 
         # The lock gets in the way of copying as implemented in clone(), which
         # is called whenever Django duplicates a StateApps before updating it.
@@ -333,9 +316,7 @@ class StateApps(Apps):
                 unrendered_models = new_unrendered_models
 
     def clone(self):
-        """
-        Return a clone of this registry, mainly used by the migration framework.
-        """
+        """Return a clone of this registry."""
         clone = StateApps([], {})
         clone.all_models = copy.deepcopy(self.all_models)
         clone.app_configs = copy.deepcopy(self.app_configs)
@@ -364,11 +345,11 @@ class StateApps(Apps):
             pass
 
 
-class ModelState(object):
+class ModelState:
     """
-    Represents a Django Model. We don't use the actual Model class
-    as it's not designed to have its options changed - instead, we
-    mutate this one and then render it into a Model as required.
+    Represent a Django Model. Don't use the actual Model class as it's not
+    designed to have its options changed - instead, mutate this one and then
+    render it into a Model as required.
 
     Note that while you are allowed to mutate .fields, you are not allowed
     to mutate the Field instances inside there themselves - you must instead
@@ -377,7 +358,7 @@ class ModelState(object):
 
     def __init__(self, app_label, name, fields, options=None, bases=None, managers=None):
         self.app_label = app_label
-        self.name = force_text(name)
+        self.name = name
         self.fields = fields
         self.options = options or {}
         self.options.setdefault('indexes', [])
@@ -417,9 +398,7 @@ class ModelState(object):
 
     @classmethod
     def from_model(cls, model, exclude_rels=False):
-        """
-        Feed me a model, get a ModelState representing it out.
-        """
+        """Given a model, return a ModelState representing it."""
         # Deconstruct the fields
         fields = []
         for field in model._meta.local_fields:
@@ -427,7 +406,7 @@ class ModelState(object):
                 continue
             if isinstance(field, OrderWrt):
                 continue
-            name = force_text(field.name, strings_only=True)
+            name = field.name
             try:
                 fields.append((name, field.clone()))
             except TypeError as e:
@@ -438,7 +417,7 @@ class ModelState(object):
                 ))
         if not exclude_rels:
             for field in model._meta.local_many_to_many:
-                name = force_text(field.name, strings_only=True)
+                name = field.name
                 try:
                     fields.append((name, field.clone()))
                 except TypeError as e:
@@ -463,12 +442,11 @@ class ModelState(object):
                 elif name == "indexes":
                     indexes = [idx.clone() for idx in model._meta.indexes]
                     for index in indexes:
-                        index.set_name_with_model(model)
+                        if not index.name:
+                            index.set_name_with_model(model)
                     options['indexes'] = indexes
                 else:
                     options[name] = model._meta.original_attrs[name]
-        # Force-convert all options to text_type (#23226)
-        options = cls.force_text_recursive(options)
         # If we're ignoring relationships, remove all field-listing model
         # options (that option basically just means "make a stub model")
         if exclude_rels:
@@ -505,15 +483,14 @@ class ModelState(object):
             for base in flattened_bases
         )
         # Ensure at least one base inherits from models.Model
-        if not any((isinstance(base, six.string_types) or issubclass(base, models.Model)) for base in bases):
+        if not any((isinstance(base, str) or issubclass(base, models.Model)) for base in bases):
             bases = (models.Model,)
 
         managers = []
         manager_names = set()
         default_manager_shim = None
         for manager in model._meta.managers:
-            manager_name = force_text(manager.name)
-            if manager_name in manager_names:
+            if manager.name in manager_names:
                 # Skip overridden managers.
                 continue
             elif manager.use_in_migrations:
@@ -529,8 +506,8 @@ class ModelState(object):
                     default_manager_shim = new_manager
             else:
                 continue
-            manager_names.add(manager_name)
-            managers.append((manager_name, new_manager))
+            manager_names.add(manager.name)
+            managers.append((manager.name, new_manager))
 
         # Ignore a shimmed default manager called objects if it's the only one.
         if managers == [('objects', default_manager_shim)]:
@@ -546,29 +523,11 @@ class ModelState(object):
             managers,
         )
 
-    @classmethod
-    def force_text_recursive(cls, value):
-        if isinstance(value, six.string_types):
-            return force_text(value)
-        elif isinstance(value, list):
-            return [cls.force_text_recursive(x) for x in value]
-        elif isinstance(value, tuple):
-            return tuple(cls.force_text_recursive(x) for x in value)
-        elif isinstance(value, set):
-            return set(cls.force_text_recursive(x) for x in value)
-        elif isinstance(value, dict):
-            return {
-                cls.force_text_recursive(k): cls.force_text_recursive(v)
-                for k, v in value.items()
-            }
-        return value
-
     def construct_managers(self):
-        "Deep-clone the managers using deconstruction"
+        """Deep-clone the managers using deconstruction."""
         # Sort all managers by their creation counter
         sorted_managers = sorted(self.managers, key=lambda v: v[1].creation_counter)
         for mgr_name, manager in sorted_managers:
-            mgr_name = force_text(mgr_name)
             as_manager, manager_path, qs_path, args, kwargs = manager.deconstruct()
             if as_manager:
                 qs_class = import_string(qs_path)
@@ -578,26 +537,29 @@ class ModelState(object):
                 yield mgr_name, manager_class(*args, **kwargs)
 
     def clone(self):
-        "Returns an exact copy of this ModelState"
+        """Return an exact copy of this ModelState."""
         return self.__class__(
             app_label=self.app_label,
             name=self.name,
             fields=list(self.fields),
+            # Since options are shallow-copied here, operations such as
+            # AddIndex must replace their option (e.g 'indexes') rather
+            # than mutating it.
             options=dict(self.options),
             bases=self.bases,
             managers=list(self.managers),
         )
 
     def render(self, apps):
-        "Creates a Model object from our current state into the given apps"
+        """Create a Model object from our current state into the given apps."""
         # First, make a Meta object
         meta_contents = {'app_label': self.app_label, "apps": apps}
         meta_contents.update(self.options)
-        meta = type(str("Meta"), tuple(), meta_contents)
+        meta = type("Meta", (), meta_contents)
         # Then, work out our bases
         try:
             bases = tuple(
-                (apps.get_model(base) if isinstance(base, six.string_types) else base)
+                (apps.get_model(base) if isinstance(base, str) else base)
                 for base in self.bases
             )
         except LookupError:
@@ -609,18 +571,8 @@ class ModelState(object):
 
         # Restore managers
         body.update(self.construct_managers())
-
-        with warnings.catch_warnings():
-            warnings.filterwarnings(
-                "ignore", "Managers from concrete parents will soon qualify as default managers",
-                RemovedInDjango20Warning)
-
-            # Then, make a Model object (apps.register_model is called in __new__)
-            return type(
-                str(self.name),
-                bases,
-                body,
-            )
+        # Then, make a Model object (apps.register_model is called in __new__)
+        return type(self.name, bases, body)
 
     def get_field_by_name(self, name):
         for fname, field in self.fields:
@@ -648,6 +600,3 @@ class ModelState(object):
             (self.bases == other.bases) and
             (self.managers == other.managers)
         )
-
-    def __ne__(self, other):
-        return not (self == other)
